;=========================================================================
;
; Liaisons avec EXCEL V1.04
;
; (C) Patrick_35
;
;=========================================================================

(defun c:ixl(/ 	classeur cle doc ixldia liste_fichiers_ouvert res s wks xl xls
		affiche_fichier ajouter_feuille ajouter_fichier changer_feuille
		changer_fichier	choix_feuille erreur_fichier fermer_excel
		gestion_affichage_boite	lancer_excel liste_feuilles liste_fichiers
		mise_a_jour msgbox position_feuille position_fichier supprimer_feuille
		supprimer_fichier verifier_excel *errixl*)

;-------------------------------------------------------------------------
; Grer les erreurs
;-------------------------------------------------------------------------

  (defun *errixl* (msg)
    (or (member (strcase msg) '("FUNCTION CANCELLED" ""QUIT / EXIT ABORT"" "FONCTION ANNULEE" "QUITTER / SORTIR ABANDON"))
      (princ (strcat "\nErreur : " msg))
    )
    (vla-endundomark doc)
    (setq *error* s)
    (princ)
  )

;-------------------------------------------------------------------------
; Afficher un message via une boite de dialogue
;-------------------------------------------------------------------------

  (defun MsgBox (Titre Bouttons Message / Reponse WshShell)
    (acad-push-dbmod)
    (setq WshShell (vlax-create-object "WScript.Shell"))
    (setq Reponse (vlax-invoke WshShell 'Popup Message 0 Titre (itoa Bouttons)))
    (vlax-release-object WshShell)
    (acad-pop-dbmod)
    Reponse
  )

;-------------------------------------------------------------------------
; Lancer une liaison avec Excel
;-------------------------------------------------------------------------

  (defun lancer_excel(/ sel)
    (setq xl  (vlax-get-or-create-object "Excel.Application"))
    (setq wks (vlax-get xl 'Workbooks))
    (vlax-for sel wks
      (setq liste_fichiers_ouvert (append liste_fichiers_ouvert (list (strcase (vlax-get sel 'fullname)))))
    )
  )

;-------------------------------------------------------------------------
; Selectionner une feuille Excel
;-------------------------------------------------------------------------

  (defun choix_feuille(cla / feu pos res)
    (setq res 0)
    (new_dialog "ixl" ixldia "")
    (start_list "feuille")
    (vlax-for feu cla
      (add_list (vlax-get feu 'name))
      (if (eq (vlax-get feu 'name) (vlax-get (vlax-get (vlax-get cla 'parent) 'activesheet) 'name))
	(setq pos (itoa res))
      )
      (setq res (1+ res))
    )
    (end_list)
    (set_tile "feuille" pos)
    (mode_tile "feuille" 2)
    (action_tile "feuille" "(setq pos $value)")
    (action_tile "accept"  "(done_dialog 1)")
    (action_tile "cancel"  "(done_dialog 0)")
    (setq res (start_dialog))
    (if (eq res 1)
      (setq feu (vlax-get (vlax-get-property cla 'item (1+ (atoi pos))) 'name))
    )
    feu
  )

;-------------------------------------------------------------------------
; Fermer la liaison avec Excel
;-------------------------------------------------------------------------

  (defun fermer_excel(/ ok sel)
    (if (not (member (strcase (vlax-get xls 'fullname)) liste_fichiers_ouvert))
      (vlax-invoke-method xls 'close :vlax-false)
    )
    (foreach sel (list xl wks xls classeur)
      (vlax-release-object sel)
    )
    (setq xl nil wks nil xls nil classeur nil)
    (gc)(gc)
  )

;-------------------------------------------------------------------------
; Recherche la liste des fichiers dans la base
;-------------------------------------------------------------------------

  (defun liste_fichiers(/ fic lst tbl)
    (mapcar '(lambda (x) (vl-filename-base (cadr x))) (reverse (vlax-ldata-list cle)))
  )

;-------------------------------------------------------------------------
; Recherche la liste des feuilles dans la base
;-------------------------------------------------------------------------

  (defun liste_feuilles(/ lst sel)
    (mapcar 'car (cdr (vlax-ldata-get cle position_fichier_excel_ixl)))
  )

;-------------------------------------------------------------------------
; Affiche le Rpertoire o se situe le fichier Excel
;-------------------------------------------------------------------------

  (defun affiche_fichier(/ sel txt)
    (set_tile "fichier" position_fichier_excel_ixl)
    (setq txt (vl-filename-directory (car (vlax-ldata-get cle position_fichier_excel_ixl))))
    (if (> (strlen txt) 31)
      (setq txt (strcat (substr txt 1 3) "...." (substr txt (- (strlen txt) 30))))
    )
    (set_tile "chemin" txt)
  )

;-------------------------------------------------------------------------
; Gestion de la boite de dialogue principale
;-------------------------------------------------------------------------

  (defun gestion_affichage_boite(chg / lst sel)
    (or effacer_blocs_excel_ixl
      (setq effacer_blocs_excel_ixl "1")
    )
    (set_tile "eff" effacer_blocs_excel_ixl)
    (if (not (vlax-ldata-list cle))
      (foreach sel (list "fichier" "sfichier" "cfichier" "chemin" "gfeuille" "maj" "eff")
	(mode_tile sel 1)
      )
      (progn
	(start_list "fichier")
	  (mapcar 'add_list (liste_fichiers))
	(end_list)
	(or position_fichier_excel_ixl
	  (setq position_fichier_excel_ixl "0")
	)
	(or position_feuille_excel_ixl
	  (setq position_feuille_excel_ixl "0")
	)
	(affiche_fichier)
	(if (cadr (vlax-ldata-get cle position_fichier_excel_ixl))
	  (progn
	    (foreach sel (list "feuille" "sfeuille" "cfeuille" "maj")
	      (mode_tile sel 0)
	    )
	    (if (setq lst (cdr (nth (1+ (atoi position_feuille_excel_ixl)) (vlax-ldata-get cle position_fichier_excel_ixl))))
	      (foreach sel (list "eff")
		(mode_tile sel 0)
	      )
	      (foreach sel (list "eff")
		(mode_tile sel 1)
	      )
	    )
	    (start_list "feuille")
	      (mapcar 'add_list (liste_feuilles))
	    (end_list)
	    (if chg
	      (setq position_feuille_excel_ixl "0")
	    )
	    (mode_tile "maj" 2)
	  )
	  (progn
	    (foreach sel (list "feuille" "sfeuille" "cfeuille" "maj" "eff")
	      (mode_tile sel 1)
	    )
	    (setq position_feuille_excel_ixl "0")
	    (mode_tile "accept" 2)
	  )
	)
	(set_tile "feuille" position_feuille_excel_ixl)
      )
    )
  )

;-------------------------------------------------------------------------
; Mise  jour de l'affichage en fonction de la slection du fichier
;-------------------------------------------------------------------------

  (defun position_fichier(val)
    (setq position_fichier_excel_ixl val position_feuille_excel_ixl "0")
    (gestion_affichage_boite T)
  )

;-------------------------------------------------------------------------
; Mise  jour de l'affichage en fonction de la slection de la feuille
;-------------------------------------------------------------------------

  (defun position_feuille(val)
    (setq position_feuille_excel_ixl val)
    (gestion_affichage_boite nil)
  )

;-------------------------------------------------------------------------
; Ajouter un fichier
;-------------------------------------------------------------------------

  (defun ajouter_fichier(/ fic ok sel)
    (or chemin_fichier_excel
      (setq chemin_fichier_excel (getvar "dwgprefix"))
    )
    (if (setq fic (getfiled "Veuillez slectionner un fichier Excel" chemin_fichier_excel "xls" 16))
      (progn
	(setq chemin_fichier_excel (strcat (vl-filename-directory fic) "\\"))
	(foreach sel (vlax-ldata-list cle)
	  (if (eq (strcase fic) (strcase (cadr sel)))
	    (setq ok T position_fichier_excel_ixl (car sel))
	  )
	)
	(if ok
	  (progn
	    (affiche_fichier)
	    (msgbox "LXL" 48 (strcat "Le fichier \"" (vl-filename-base fic) ".xls\" est dj rfrenc dans le dessin."))
	  )
	  (progn
	    (setq position_fichier_excel_ixl (itoa (length (vlax-ldata-list cle))))
	    (vlax-ldata-put cle position_fichier_excel_ixl (list fic))
	    (if (eq position_fichier_excel_ixl "0")
	      (progn
		(foreach sel (list "fichier" "sfichier" "cfichier" "chemin" "gfeuille")
		  (mode_tile sel 0)
		)
		(foreach sel (list "feuille" "sfeuille" "cfeuille")
		  (mode_tile sel 1)
		)
	      )
	    )
	    (start_list "fichier" 2)
	      (add_list (vl-filename-base fic))
	    (end_list)
	    (set_tile "fichier" position_fichier_excel_ixl)
	  )
	)
	(gestion_affichage_boite nil)
      )
    )
  )

;-------------------------------------------------------------------------
; Supprimer un fichier
;-------------------------------------------------------------------------

  (defun supprimer_fichier(/ i n ok tbl)
    (setq tbl (vlax-ldata-get cle position_fichier_excel_ixl) i 0 n 0)
    (if (cdr tbl)
      (if (eq (msgbox "LXL" (+ 48 1 256) (strcat "Des feuilles Excel sont lis avec le fichier\n\"" (car tbl) "\"\nDsirez-vous le supprimer ?")) 2)
	(setq ok T)
      )
    )
    (if (not ok)
      (progn
	(while (setq tbl (vlax-ldata-get cle (itoa n)))
	  (if (not (eq (itoa n) position_fichier_excel_ixl))
	    (progn
	      (vlax-ldata-put cle (itoa i) tbl)
	      (setq i (1+ i))
	    )
	  )
	  (setq n (1+ n))
	)
	(vlax-ldata-delete cle (itoa (1- n)))
	(if (> (atoi position_fichier_excel_ixl) 0)
	  (setq position_fichier_excel_ixl (itoa (1- (atoi position_fichier_excel_ixl))))
	)
	(gestion_affichage_boite nil)
      )
    )
  )

;-------------------------------------------------------------------------
; Changer un fichier
;-------------------------------------------------------------------------

  (defun changer_fichier(/ fic ok sel)
    (or chemin_fichier_excel
      (setq chemin_fichier_excel (getvar "dwgprefix"))
    )
    (if (setq fic (getfiled "Veuillez slectionner un fichier Excel" chemin_fichier_excel "xls" 16))
      (progn
	(setq chemin_fichier_excel (strcat (vl-filename-directory fic) "\\"))
	(foreach sel (vlax-ldata-list cle)
	  (if (eq (strcase fic) (strcase (cadr sel)))
	    (setq ok T position_fichier_excel_ixl (car sel))
	  )
	)
	(if ok
	  (progn
	    (affiche_fichier)
	    (msgbox "LXL" 48 (strcat "Le fichier \"" (vl-filename-base fic) ".xls\" est dj rfrenc dans le dessin."))
	  )
	  (progn
	    (setq sel (cons fic (cdr (vlax-ldata-get cle position_fichier_excel_ixl))))
	    (vlax-ldata-put cle position_fichier_excel_ixl sel)
	    (gestion_affichage_boite nil)
	  )
	)
      )
    )
  )

;-------------------------------------------------------------------------
; Message d'erreur de fichier non trouv
;-------------------------------------------------------------------------

  (defun erreur_fichier()
    (msgbox "LXL" 16 (strcat "Le fichier \""
			     (vl-filename-base (car (vlax-ldata-get cle position_fichier_excel_ixl)))
			     (vl-filename-extension (car (vlax-ldata-get cle position_fichier_excel_ixl)))
			     "\" est introuvable.\nSoit il a t renomm, supprim ou dplac du rpertoire\n\""
			     (vl-filename-directory (car (vlax-ldata-get cle position_fichier_excel_ixl)))
			     "\"."
		     )
    )
  )

;-------------------------------------------------------------------------
; Verifie si la feuille est actif dans Excel, si non l'ouvre
;-------------------------------------------------------------------------

  (defun verifier_excel()
    (or xl
      (lancer_excel)
    )
    (if xls
      (if (not (eq (strcase (vlax-get xls 'fullname)) (strcase (car (vlax-ldata-get cle position_fichier_excel_ixl)))))
	(progn
	  (if (not (member (strcase (vlax-get xls 'fullname)) liste_fichiers_ouvert))
	    (vlax-invoke-method xls 'close :vlax-false)
	  )
	  (vlax-release-object xls)
	  (setq xls nil)
	)
      )
    )
    (vlax-for sel wks
      (if (eq (strcase (vlax-get sel 'fullname)) (strcase (car (vlax-ldata-get cle position_fichier_excel_ixl))))
	(setq xls sel)
      )
    )
    (or xls
      (setq xls (vlax-invoke wks 'open (car (vlax-ldata-get cle position_fichier_excel_ixl))))
    )
    (setq classeur (vlax-get xls 'sheets))
  )

;-------------------------------------------------------------------------
; Ajouter une feuille Excel
;-------------------------------------------------------------------------

  (defun ajouter_feuille(/ feu n lance lst ok sel)
    (if (findfile (car (vlax-ldata-get cle position_fichier_excel_ixl)))
      (progn
	(verifier_excel)
	(if (setq feu (choix_feuille classeur))
	  (progn
	    (setq lst (vlax-ldata-get cle position_fichier_excel_ixl) n 0)
	    (if (cdr lst)
	      (while (setq sel (nth n (cdr lst)))
		(if (eq (strcase (car sel)) (strcase feu))
		  (setq ok T position_feuille_excel_ixl (itoa n) n (length (cdr lst)))
		)
		(setq n (1+ n))
	      )
	    )
	    (if ok
	      (msgbox "LXL" 48 (strcat "La feuille \"" feu "\" est dj rfrence dans le dessin."))
	      (progn
		(setq lst (append lst (list (list feu))))
		(vlax-ldata-put cle position_fichier_excel_ixl lst)
		(start_list "feuille" 2)
		  (add_list feu)
		(end_list)
		(foreach rec (list "feuille" "sfeuille" "cfeuille")
		  (mode_tile rec 0)
		)
		(mode_tile "att" 1)
		(setq position_feuille_excel_ixl (itoa (1- (length (cdr lst)))))
	      )
	    )
	    (gestion_affichage_boite nil)
	  )
	)
      )
      (erreur_fichier)
    )
  )

;-------------------------------------------------------------------------
; Supprimer une feuille Excel
;-------------------------------------------------------------------------

  (defun supprimer_feuille(/ lst ok tbl)
    (setq tbl (nth (1+ (atoi position_feuille_excel_ixl)) (vlax-ldata-get cle position_fichier_excel_ixl)))
    (if (cdr tbl)
      (if (eq (msgbox "LXL" (+ 48 1 256) (strcat "Des blocs sont lis avec la feuille \"" (car tbl) "\"\nDsirez-vous la supprimer ?")) 2)
	(setq ok T)
      )
    )
    (if (not ok)
      (progn
	(setq lst (vl-remove tbl (vlax-ldata-get cle position_fichier_excel_ixl)))
	(vlax-ldata-put cle position_fichier_excel_ixl lst)
	(if (> (atoi position_feuille_excel_ixl) 0)
	  (setq position_feuille_excel_ixl (itoa (1- (atoi position_feuille_excel_ixl))))
	)
	(gestion_affichage_boite nil)
      )
    )
  )

;-------------------------------------------------------------------------
; Changer de feuille Excel
;-------------------------------------------------------------------------

  (defun changer_feuille(/ feu lst n ok sel tbl)
    (if (findfile (car (vlax-ldata-get cle position_fichier_excel_ixl)))
      (progn
	(verifier_excel)
	(if (setq feu (choix_feuille classeur))
	  (progn
	    (setq lst (vlax-ldata-get cle position_fichier_excel_ixl) n 0)
	    (if (cdr lst)
	      (while (setq sel (nth n (cdr lst)))
		(if (eq (strcase (car sel)) (strcase feu))
		  (setq ok T position_feuille_excel_ixl (itoa n) n (length (cdr lst)))
		)
		(setq n (1+ n))
	      )
	    )
	    (if ok
	      (msgbox "LXL" 48 (strcat "La feuille \"" feu "\" est dj rfrence dans le dessin."))
	      (progn
		(setq tbl (nth (1+ (atoi position_feuille_excel_ixl)) (vlax-ldata-get cle position_fichier_excel_ixl))
		      lst (subst (cons feu (cdr tbl)) tbl (vlax-ldata-get cle position_fichier_excel_ixl)))
		(vlax-ldata-put cle position_fichier_excel_ixl lst)
		(gestion_affichage_boite nil)
	      )
	    )
	  )
	)
      )
      (erreur_fichier)
    )
  )

;; INCSUFF (version 1.2) -Gilles Chanteau- 15/01/08
;; Incrmente le suffixe d'une chane de caractres de la valeur spcifie.
;; Est pris en compte comme suffixe, l'ensemble des caractres [0-9] et/ou [A-Z]
;; et/ou [a-z] depuis la fin du texte, en fonction de la valeur de alpha
;;
;; Arguments
;; str : une chane avec un suffixe valide
;; inc : un entier positif
;; alpha : un entier, somme des codes binaires suivant
;; 1 pour les caractres [0-9]
;; 2 pour les caractres [A-Z]
;; 4 pour les caractres [a-z]
;;
;; Retour
;; la chane avec son suffixe incrment (ou nil si aucun suffixe valide)
;;
;; Exemples :
;; (incsuff "N002" 12 1) = "N014"
;; (incsuff "Dessin9" 1 1) = "Dessin10"
;; (incsuff "test_ZZ9" 1 3) = "test_AAA0"
;; (incsuff "test_ZZ9" 1 1) = "test_ZZ10"
;; (incsuff "12-" 1 1) = nil
;;
;; Modification (13/02/08) : codes binaires pour l'argument alpha

    (defun incsuff (str inc alpha / lst crt pas ind dep quo ret)
      (setq lst (reverse (vl-string->list str)))
      (while
	(and
	  (setq crt (car lst))
	  (cond
	    ((and (< 47 crt 58) (= 1 (logand 1 alpha)))
	      (setq pas 10
		    ind 48
	      )
	    )
	    ((and (< 64 crt 91) (= 2 (logand 2 alpha)))
	      (setq pas 26
		    ind 65
	      )
	    )
	    ((and (< 96 crt 123) (= 4 (logand 4 alpha)))
	      (setq pas 26
		    ind 97
	      )
	    )
	    ((< 0 quo)
	      (setq crt (if (= 10 pas)
			  ind
			  (1- ind)
			)
		    lst (cons (car lst) lst)
	      )
	    )
	  )
	)
	(setq dep (- crt ind)
	      quo (/ (+ dep inc) pas)
	      ret (cons (+ ind (rem (+ dep inc) pas)) ret)
	)
	(if (zerop quo)
	  (setq ret (append (reverse (cdr lst)) ret)
		lst nil
	  )
	  (if (cdr lst)
	    (setq lst (cdr lst)
		  inc quo
	    )
	    (setq lst (list ind)
		  inc (if (= 10 pas)
			quo
			(1- quo)
		      )
	    )
	  )
	)
      )
      (if ret
	(vl-list->string ret)
      )
    )

;-------------------------------------------------------------------------
; Lancer le traitement de mise  jour
;-------------------------------------------------------------------------

  (defun mise_a_jour(/	at att bloc bloc_a bloc_e bloc_x bloc_y bloc_z calque
			col ent feu esp lecture ligne lst msg obj pos tia tot
			tbl tst val wait)

    (defun lecture(cel / val) (setq aze (vlax-get-property classeur 'item (car lst)))
      (vlax-get
	(vlax-get-property
	  (vlax-get-property classeur 'item (car lst))
	  'range
	  cel
	)
	'value2
      )
    )

    (defun msg(mg)
      (repeat 24 (princ (chr 8)))
      (princ (strcat mg
		     "\nde la feuille \""
		     (car lst)
		     "\" du fichier \""
		     (cadr tbl)
		     "\"."
	     )
      )
      (princ (strcat "\nTraitement en cours... " wait))
    )

    (setq pos position_fichier_excel_ixl tia 0 tot 0 wait "|")
    (if (eq (getvar "cvport") 1)
      (setq esp (vla-get-paperspace doc))
      (setq esp (vla-get-modelspace doc))
    )
    (princ (strcat "\nTraitement en cours... " wait))
    (foreach tbl (reverse (vlax-ldata-list cle))
      (if (findfile (cadr tbl))
	(progn
	  (setq position_fichier_excel_ixl (car tbl))
	  (verifier_excel)
	  (foreach lst (cddr tbl)
	    (setq tst nil ligne 2)
	    (vlax-for feu classeur
	      (if (eq (strcase (vlax-get feu 'name)) (strcase (car lst)))
		(setq tst T)
	      )
	    )
	    (if tst
	      (progn
		(while (not (member (setq bloc (lecture (strcat "A" (itoa ligne)))) '("" 0 0.0 nil)))
		  (if (setq calque (lecture (strcat "B" (itoa ligne))))
		    (if (setq bloc_x (lecture (strcat "C" (itoa ligne))))
		      (if (setq bloc_y (lecture (strcat "D" (itoa ligne))))
			(if (setq bloc_z (lecture (strcat "E" (itoa ligne))))
			  (if (setq bloc_a (lecture (strcat "F" (itoa ligne))))
			    (if (setq bloc_e (lecture (strcat "G" (itoa ligne))))
			      (if (or (tblsearch "block" bloc)
				      (findfile (strcat bloc ".dwg"))
				  )
				(progn
				  (setq col "H" att nil)
				  (while (setq val (lecture (strcat col (itoa ligne))))
				    (setq att (cons val att)
					  col (incsuff col 1 2)
				    )
				  )
				  (or (tblsearch "block" bloc)
				    (setq bloc (findfile (strcat bloc ".dwg")))
				  )
				  (or (/= (type bloc_x) 'STR)
				    (setq bloc_x (read bloc_x))
				  )
				  (or (/= (type bloc_y) 'STR)
				    (setq bloc_y (read bloc_y))
				  )
				  (or (/= (type bloc_z) 'STR)
				    (setq bloc_z (read bloc_z))
				  )
				  (or (/= (type bloc_a) 'STR)
				    (setq bloc_a (read bloc_a))
				  )
				  (or (/= (type bloc_e) 'STR)
				    (setq bloc_e (read bloc_e))
				  )
				  (if (vl-catch-all-error-p (setq ent (vl-catch-all-apply 'vla-insertblock (list esp (vlax-3d-point (list bloc_x bloc_y bloc_z)) bloc bloc_e bloc_e bloc_e bloc_a))))
				    (msg (strcat "Erreur lors de l'insertion du bloc \"" bloc "\" indiqu dans la cellule \"A" (itoa ligne) "\""))
				    (progn
				      (setq obj (cons ent obj))
				      (cond
					((eq (type calque) 'INT)
					  (setq calque (itoa calque))
					)
					((eq (type calque) 'REAL)
					  (setq calque (rtos calque))
					  (or (vl-string-search "." (vl-string-right-trim "." (vl-string-right-trim "0" calque)))
					    (setq calque (vl-string-right-trim "." (vl-string-right-trim "0" calque)))
					  )
					)
				      )
				      (or (tblsearch "layer" calque)
				        (vla-add (vla-get-layers doc) calque)
				      )
				      (if (vl-catch-all-error-p (vl-catch-all-apply 'vla-put-layer (list (car obj) calque)))
					(msg (strcat "Erreur lors de l'affectation du calque" calque " pour le bloc \"" bloc "\" indiqu dans la cellule \"A" (itoa ligne) "\""))
				      )
				      (if (setq att (reverse att))
				        (if (setq at (vlax-invoke (car obj) 'getattributes))
				          (foreach ent at
					    (if att
					      (progn
					        (vla-put-textstring ent (car att))
					        (setq att (cdr att))
					      )
					    )
				          )
				          (msg (strcat "Pas d'attributs pour le bloc \"" bloc "\" indiqu dans la cellule \"A" (itoa ligne) "\""))
				        )
				      )
				      (setq tia (1+ tia))
				    )
				  )
				)
				(msg (strcat "Impossible de trouver le bloc \"" bloc "\" indiqu dans la cellule \"A" (itoa ligne) "\""))
			      )
			      (msg (strcat "Le facteur d'chelle n'est pas indiqu dans la cellule \"G" (itoa ligne) "\""))
			    )
			    (msg (strcat "L'angle n'est pas indiqu dans la cellule \"F" (itoa ligne) "\""))
			  )
			  (msg (strcat "Le point d'insertion Z n'est pas indiqu dans la cellule \"E" (itoa ligne) "\""))
			)
			(msg (strcat "Le point d'insertion Y n'est pas indiqu dans la cellule \"D" (itoa ligne) "\""))
		      )
		      (msg (strcat "Le point d'insertion X n'est pas indiqu dans la cellule \"C" (itoa ligne) "\""))
		    )
		    (msg (strcat "Le nom du calque n'est pas indiqu dans la cellule \"B" (itoa ligne) "\""))
		  )
		  (setq ligne (1+ Ligne) tot (1+ tot))
		  (cond
		    ((eq wait "|")
		      (setq wait "/")
		    )
		    ((eq wait "/")
		      (setq wait "-")
		    )
		    ((eq wait "-")
		      (setq wait "\\")
		    )
		    (T
		      (setq wait "|")
		    )
		  )
		  (princ (strcat (chr 8) wait))(princ)
		)
		(and (cadr lst)
		     (eq effacer_blocs_excel_ixl "1")
		     (foreach ent (cadr lst)
		       (if (vlax-read-enabled-p ent)
			 (vla-delete ent)
		       )
		     )
		)
		(if obj
		  (progn
		    (setq tbl (subst (append (list (car lst)) (list obj)) lst tbl))
		    (vlax-ldata-put cle (car tbl) (cdr tbl))
		  )
		)
	      )
	      (progn
		(repeat 24 (princ (chr 8)))
		(princ (strcat	"La feuille \""
				(car lst)
				"\" du fichier \""
				(vl-filename-base (cadr tbl))
				(vl-filename-extension (cadr tbl))
				"\" est introuvable.\nElle a t renomme ou supprime."
		       )
		)
		(princ (strcat "\nTraitement en cours... " wait))
	      )
	    )
	  )
	)
	(progn
	  (repeat 24 (princ (chr 8)))
	  (princ (strcat "Le fichier \""
			 (vl-filename-base (cadr tbl))
			 (vl-filename-extension (cadr tbl))
			 "\" est introuvable.\nIl a t renomm, supprim ou dplac du rpertoire\n\""
			 (vl-filename-directory (cadr tbl))
			 "\"."
	       )
	  )
	  (princ (strcat "\nTraitement en cours... " wait))
	)
      )
    )
    (setq position_fichier_excel_ixl pos)
    (princ (strcat (chr 8) "termin. " (itoa tia) " Ligne(s) Excel traite(s) sur " (itoa tot) "."))
    (princ)
  )

;-------------------------------------------------------------------------
; Routine principale
;-------------------------------------------------------------------------

  (vl-load-com)
  (setq cle "Patrick_35_Excel_Ixl"
	doc (vla-get-activedocument (vlax-get-acad-object))
	s *error*
	*error* *errixl*
  )
  (vla-startundomark doc)
  (if (setq ixldia (findfile "ixl.dcl"))
    (progn
      (setq ixldia (load_dialog ixldia))
      (while (and (not (member res (list 0 5))))
	(new_dialog "choix" ixldia "")
	(gestion_affichage_boite nil)
	(set_tile "accept" "2")
	(action_tile "fichier"  "(position_fichier $value)")
	(action_tile "afichier" "(done_dialog 1)")
	(action_tile "sfichier" "(supprimer_fichier)")
	(action_tile "cfichier" "(done_dialog 2)")
	(action_tile "feuille"  "(position_feuille $value)")
	(action_tile "afeuille" "(done_dialog 3)")
	(action_tile "sfeuille" "(supprimer_feuille)")
	(action_tile "cfeuille" "(done_dialog 4)")
	(action_tile "eff"      "(setq effacer_blocs_excel_ixl $value)")
	(action_tile "maj"      "(done_dialog 5)")
	(action_tile "accept"   "(done_dialog 0)")
	(action_tile "cancel"   "(done_dialog 0)")
	(setq res (start_dialog))
	(cond
	  ((eq res 1)
	    (ajouter_fichier)
	  )
	  ((eq res 2)
	    (changer_fichier)
	  )
	  ((eq res 3)
	    (ajouter_feuille)
	  )
	  ((eq res 4)
	    (changer_feuille)
	  )
	  ((eq res 5)
	    (mise_a_jour)
	  )
	)
      )
      (unload_dialog ixldia)
    )
    (msgbox "IXL" 16 "Fichier IXL.DCL introuvable.")
  )
  (if xl
    (fermer_excel)
  )
  (vla-endundomark doc)
  (setq *error* s)
  (princ)
)

(setq nom_lisp "IXL")
(if (/= app nil)
  (if (= (strcase (substr app (1+ (- (strlen app) (strlen nom_lisp))) (strlen nom_lisp))) nom_lisp)
    (princ (strcat "..." nom_lisp " charg."))
    (princ (strcat "\n" nom_lisp ".LSP Charg.....Tapez " nom_lisp " pour l'xecuter.")))
  (princ (strcat "\n" nom_lisp ".LSP Charg......Tapez " nom_lisp " pour l'xecuter.")))
(setq nom_lisp nil)
(princ)